/**
 * \file
 * ${moduleGroupNameLowerCase}_host_interface.h
 *
 * \defgroup
 * i2c_host_interface
 *
 * \brief
 * This file contains API prototypes and other data types for ${moduleGroupNameUpperCase} driver.
 *
 * \version
 * I2C Host Driver Version ${hostDriverVersion}
 */

${disclaimer}

#ifndef ${moduleGroupNameUpperCase}_HOST_INTERFACE_H
#define ${moduleGroupNameUpperCase}_HOST_INTERFACE_H
/**
  Section: Included Files
*/
#include <stdbool.h>
#include <stdint.h>
#include <xc.h>
#include "${moduleGroupNameLowerCase}_host_types.h"
    
/**
  ${moduleGroupNameUpperCase}_HOST_INTERFACE

  @Description
    Structure containing the function pointers of ${moduleGroupNameUpperCase} driver.
 */
typedef struct
{   
    void (*Initialize)(void);
    void (*Deinitialize)(void);
    bool (*Write)(uint16_t address, uint8_t *data, size_t dataLength);
    bool (*Read)(uint16_t address, uint8_t *data, size_t dataLength);
    bool (*WriteRead)(uint16_t address, uint8_t *writeData, size_t writeLength, uint8_t *readData, size_t readLength);
    bool (*TransferSetup)(struct ${moduleGroupNameUpperCase}_TRANSFER_SETUP* setup, uint32_t srcClkFreq);
    ${moduleGroupNameLowerCase}_host_error_t (*ErrorGet)(void);
    bool (*IsBusy)(void);
    void (*CallbackRegister)(void (*callback)(void));
    void (*Tasks)(void);
}${moduleGroupNameLowerCase}_host_interface_t;


/**
 * @ingroup ${moduleGroupNameLowerCase}_host
 * @example ${moduleGroupNameLowerCase}_host_interface_example_code
 * @brief In this I2C Host example, a byte of data is written in to EEPROM and read it back.
 *        Case 1: Data 0xAA is written in to EEPROM at location 0x0010 and read the same back, using I2C0_Write() and I2C0_WriteRead() functions.
 *        Case 2: Data 0x55 is written in to EEPROM at location 0x0020 and read the same back using I2C0_Write() and I2C0_Read() functions.
 * @code
 * 
 * void main(void)
 * {
 *    uint8_t transmitData[10];
 *    uint8_t writeLength;
 *    uint8_t readLength;
 *    uint8_t writeData;
 *    uint8_t readData;
 *    uint8_t eepromAddr = 0x50; // 7-bit EEPROM address
 *
 *    // Initializes Clock, Pins, Interrupts and I2C host
 *    SYSTEM_Initialize();
<#if isInterruptDriven>
 *    // Case 1:
 *    transmitData[0] = 0x00;  // load MSB of EEPROM location
 *    transmitData[1] = 0x10;  // load LSB of EEPROM location
 *    transmitData[2] = 0xAA;  // load data
 *    writeLength = 3;  // 2 bytes of location address + 1 byte data
 *
 *    if(!${componentNameLowerCase}_host_interface.Write(eepromAddr, transmitData, writeLength))
 *    {
 *        // I2C bus busy, retry later
 *    }
 *
 *    // Confirm write operation completed and check for error
 *    if(!${componentNameLowerCase}_host_interface.IsBusy())
 *    {
 *        if ( ${componentNameLowerCase}_host_interface.ErrorGet() == ${moduleGroupNameUpperCase}_ERROR_NONE)
 *        {
 *            // Write operation is successful
 *        }
 *        else
 *        {
 *            // Error handling
 *        }
 *    }
 *
 *    writeLength = 2; // 2 bytes of location address
 *    readLength = 1; // 1 byte read
 *    if(${componentNameLowerCase}_host_interface.WriteRead(eepromAddr, transmitData, writeLength, readData , readLength))
 *    {
 *        // I2C bus busy, retry later
 *    }
 *
 *    // Confirm write operation completed and check for error
 *    if(!${componentNameLowerCase}_host_interface.IsBusy())
 *    {
 *         if ( ${componentNameLowerCase}_host_interface.ErrorGet() == ${moduleGroupNameUpperCase}_ERROR_NONE)
 *         {
 *            // WriteRead operation is successful
 *         }
 *         else
 *         {
 *            // Error handling
 *         }
 *    }
 *
 *    // Case 2:
 *    transmitData[0] = 0x00;  // load MSB of EEPROM location
 *    transmitData[1] = 0x20;  // load LSB of EEPROM location
 *    transmitData[2] = 0x55;  // load data
 *    writeLength = 2 + 1; // 2 bytes of location address + 1 byte data
 *    if(!${componentNameLowerCase}_host_interface.Write(eepromAddr, transmitData, writeLength))
 *    {
 *        // I2C bus busy, retry later
 *    }
 *
 *    // Confirm write operation completed and check for error
 *    if(!${componentNameLowerCase}_host_interface.IsBusy())
 *    {
 *        if ( ${componentNameLowerCase}_host_interface.ErrorGet() == ${moduleGroupNameUpperCase}_ERROR_NONE)
 *        {
 *            // Write operation is successful
 *        }
 *        else
 *        {
 *            // Error handling
 *        }
 *    }
 *
 *    writeLength = 2; // 2 bytes of location address
 *    if(!${componentNameLowerCase}_host_interface.Write(eepromAddr, transmitData, writeLength))
 *    {
 *        // I2C bus busy, retry later
 *    }
 *
 *    // Confirm write operation completed and check for error
 *    if(!${componentNameLowerCase}_host_interface.IsBusy())
 *    {
 *        if ( ${componentNameLowerCase}_host_interface.ErrorGet() == ${moduleGroupNameUpperCase}_ERROR_NONE)
 *        {
 *            // Write operation is successful
 *        }
 *        else
 *        {
 *            // Error handling
 *        }
 *    }
 *
 *    readLength = 1; // 1 byte read
 *    if(!${componentNameLowerCase}_host_interface.Read(eepromAddr, readData, readLength)))
 *    {
 *        // I2C bus busy, retry later
 *    }
 *
 *    // Confirm write operation completed and check for error
 *    if(!${componentNameLowerCase}_host_interface.IsBusy())
 *    {
 *        if ( ${componentNameLowerCase}_host_interface.ErrorGet() == ${moduleGroupNameUpperCase}_ERROR_NONE)
 *        {
 *            // Read operation is successful
 *        }
 *        else
 *        {
 *            // Error handling
 *        }
 *    }
 *
 <#else>
 *    // Case 1:
 *    transmitData[0] = 0x00;  // load MSB of EEPROM location
 *    transmitData[1] = 0x10;  // load LSB of EEPROM location
 *    transmitData[2] = 0xAA;  // load data
 *    writeLength = 3; // 2 bytes of location address + 1 byte data
 *
 *    if ( ${componentNameLowerCase}_host_interface.Write(eepromAddr, transmitData, writeLength))
 *    {
 *        while ( ${componentNameLowerCase}_host_interface.IsBusy())
 *        {
 *            ${componentNameLowerCase}_host_interface.Tasks();
 *        }
 *
 *        if (  ${componentNameLowerCase}_host_interface.ErrorGet() == ${moduleGroupNameUpperCase}_ERROR_NONE)
 *        {
 *            // Write operation is successful
 *        }
 *        else
 *        {
 *            // Error handling
 *        }
 *    }
 *
 *    writeLength = 2; // 2 bytes of location address
 *    readLength = 1; // 1 byte read
 *    if (${componentNameLowerCase}_host_interface.WriteRead(eepromAddr, transmitData, writeLength, readData , readLength))
 *    {
 *        while ( ${componentNameLowerCase}_host_interface.IsBusy())
 *        {
 *            ${componentNameLowerCase}_host_interface.Tasks();
 *        }
 *
 *        if (  ${componentNameLowerCase}_host_interface.ErrorGet() == ${moduleGroupNameUpperCase}_ERROR_NONE)
 *        {
 *            // WriteRead operation is successful
 *        }
 *        else
 *        {
 *            // Error handling
 *        }
 *    }
 *
 *    // Case 2:
 *    transmitData[0] = 0x00;  // load MSB of EEPROM location
 *    transmitData[1] = 0x20;  // load LSB of EEPROM location
 *    transmitData[2] = 0x55;  // load data
 *    writeLength = 3; // 2 bytes of location address + 1 byte data
 *    if (${componentNameLowerCase}_host_interface.Write(eepromAddr, transmitData, writeLength))
 *    {
 *        while (${componentNameLowerCase}_host_interface.IsBusy())
 *        {
 *            ${componentNameLowerCase}_host_interface.Tasks();
 *        }
 *
 *        if ( ${componentNameLowerCase}_host_interface.ErrorGet() == ${moduleGroupNameUpperCase}_ERROR_NONE)
 *        {
 *            // Write operation is successful
 *        }
 *        else
 *        {
 *            // Error handling
 *        }
 *    }
 *
 *    writeLength = 2; // 2 bytes of location address
 *    if (${componentNameLowerCase}_host_interface.Write(eepromAddr, transmitData, writeLength))
 *    {
 *        while (${componentNameLowerCase}_host_interface.IsBusy())
 *        {
 *            ${componentNameLowerCase}_host_interface.Tasks();
 *        }
 *
 *        if ( ${componentNameLowerCase}_host_interface.ErrorGet() == ${moduleGroupNameUpperCase}_ERROR_NONE)
 *        {
 *            // Write operation is successful
 *        }
 *        else
 *        {
 *            // Error handling
 *        }
 *    }
 *
 *    readLength = 1; // 1 byte read
 *    if (${componentNameLowerCase}_host_interface.Read(eepromAddr, readData, readLength))
 *    {
 *        while (${componentNameLowerCase}_host_interface.IsBusy())
 *        {
 *            ${componentNameLowerCase}_host_interface.Tasks();
 *        }
 *
 *        if ( ${componentNameLowerCase}_host_interface.ErrorGet() == ${moduleGroupNameUpperCase}_ERROR_NONE)
 *        {
 *            // Read operation is successful
 *        }
 *        else
 *        {
 *            // Error handling
 *        }
 *    }
 *
</#if>
 *     while (1)
 *     {
 *     }
 * }
 * @endcode
*/


#endif // end of ${moduleGroupNameUpperCase}_HOST_INTERFACE_H